% this script solves a game in extensive form by backward induction
% user has to input the following info about the game 
%       number of players
%       list of vertices (and division into terminal and nonterminal)
%       the player partition
%       game tree (encoded here as the list of successors)
%       rewards at terminal vertices

%% User enters inputs of the game
nPlayers = 2; % number of players

nt_vert=1:7;  % specify non terminal vertices (consecutive numbers)
t_vert=8:15;  % specify terminal vertices (consecutive numbers)

% specify the player partition
% PPartition(v)=1 if player 1 moves at v, 2 if player 2 moves ...
PPartition = [1 2 2 1 1 1 1]; 

% list numbers of successors for each vertex 
% successors do not have to be of same length
sc = {[2 3], [4 5], [6 7], [8 9], [10 11], [12 13], [14 15]};

% specify the rewards at terminal vertices
% reward(v,p) is reward at vertex v for player p
reward(t_vert, 1:nPlayers) = [[8 1]; [7 7]; [2 0]; [3 3]; ...
                              [4 1]; [0 3]; [5 2]; [1 4]];

%% Automatic rest of init procedures
vertices = [nt_vert t_vert]; % creates list of all vertices
opt_choice = NaN(1,length(nt_vert)); % initialize the optimal choice
% opt_choice(v) will be the optimal choice to go from vertex v
%       NaN stands for not a number. 
%       if opt_choice(v) is NaN, it means no choice made yet at v

un_vert = nt_vert(isnan(opt_choice)); % vertices where choice is not known 

% initializes the rewards for nonterminal vertices
reward(nt_vert, 1:nPlayers)=NaN(length(nt_vert), nPlayers); 
% NaN specify that the reward is not known yet


%% Actual analysis
while ~isempty(un_vert) % while there are undecided vertices
    for v=un_vert % go over all such vertices
        successors=sc{v}; % get all successors of vertex v
        possible_rewards = reward(successors,PPartition(v)); % possible reward for players on the move
        % if we know the rewards for all of the successors of v
        if ~any(isnan(possible_rewards)) 
            % (if there is no NaN in the rewards of successors)
            [temp, Index]=max(possible_rewards); % get index of the maximal
            opt_choice(v)=successors(Index); % update optimal choice
            reward(v,:)=reward(opt_choice(v), :); % update reward at v
        end
    end
    un_vert = nt_vert(isnan(opt_choice)); % update undecided vertices
end

%% Outputs
disp('Optimal choices and payoffs are as follows');
for v=nt_vert
    disp(['From vertex ' num2str(v) ' go to vertex ' num2str(opt_choice(v)) ...
            ' to get rewards ' num2str(reward(v,:)) ]);
end;

