% stochastic foraging game
% birds look for food in a given time interval 0 ... maxT
% at the end of the interval they need to have enough resources to survive
% they can be killed during foraging and may not always find food
% this script outputs the optimal behaviour (stay or forage)

%% user defined parameters
z = 0.01;  % probability of being killed during foraging
c = 0.9;   % probability of increasing resources during foraging
           % c above 0.9 could cause rounding errors for small t 
           % especially when maxT is large
maxT = 20; % maximal time for foraging
maxR = 20; % maximal value of resources
threshold = 15; % bird must have above threshold to survive


%% init of probability to live
P = double(zeros(maxR+1, maxT+1)); % init value for allocation 
% set boundary conditions:
% at the end, only those with enough resources will survive, others will die
P(:, maxT+1) = ((1:maxR+1)>threshold); 
% at any moment, having more than maxR means to survive
P(maxR+1, :) = 1;

hold on; % we will plot a graph at the same time as we calculate
xlabel('Time','FontSize',16)
ylabel('Resource level','FontSize',16)
axis([0 maxT+1 0 maxR+1])

%% backward induction
for t=(maxT):-1:1 % start at the last time to make a decision and move back
    for r=1:maxR  % for all possible resource values at time t
        x_coord = [t-1/2 t-1/2 t+1/2 t+1/2]; % coordinates for drawing
        y_coord = [r-1/2 r+1/2 r+1/2 r-1/2];
        %r
        %t
        stay = P(r,t+1); % if stay, one would get this much 
        forage = (P(r+1,t+1)*c + stay*(1-c))*(1-z); % reward for foraging
        if forage > stay % if foraging is better
            P(r,t) = forage;
            fill(x_coord, y_coord, 'green');
        elseif forage == stay % does not matter whether to forage or not
            P(r,t) = forage;
            if forage == 0 % it does not survive
                fill(x_coord, y_coord, 'black');
            else % This sometimes happens also due to rounding errors
                 fill(x_coord, y_coord, 'yellow');
            end;
        else % staying is better
            P(r,t) = stay;
            fill(x_coord, y_coord, 'red');
        end;
    end;
end;


