function SIS
% Uses Gillespie stochastic simulation algorithm to simulate 
% SIS epidemics. It plots stochastics dynamics output together 
% with solution of the ODE

% in order for this script to work, the function Gillespie 
% from the script for Chapter 18 needs to be saved as 
% a separate file in the same folder as this script

%%  User defined input
N = 100;        % total population size
I = 1;          % initial size of infected population
time = 50;      % max time of observation
beta = 1;       % rate of interactions (inds per unit of time)
g = 0.5;        % recovery rate

%% prepare input for Gillespie algorithm
function a=Propensities(x)
    % there are 2 processes with 2 compartments S, I
    % S --> I with speed beta*S*I/N
    % I --> S with speed g*I
    a(1) = beta*x(1)*x(2)/(x(1)+x(2)); % S becomes I
    a(2) = g * x(2);                   % I becomes S
end;

e = [1, 0;    % S entering process 1, 2
     0, 1];   % I entering process 1, 2

p = [0, 1;    % S resulting from process 1, 2
     1, 0];   % I resulting from process 1, 2
     
Pop_init = [N-I; I];  %S; I

%% call the Gillespie algorithm
[Pop, T]= Gillespie(Pop_init, @Propensities, e, p, time, 10000);

%% Solve ODE numerically
    function dv = SISdynamics(t,v)
        dv = v.*(beta*(1-v)-g); % SIS dynamics for v=I/N
    end
[T_ODE,V] = ode45(@SISdynamics,[0 time], I/N); 

%% Plots
% the following few lines prepare for stair like drawing of the results
m=size(Pop,2); % number of steps (plus 1) in the simulation
I_stairs=zeros(1, 2*m-1);
t_stairs=zeros(1, 2*m-1);
I_stairs(1:2:2*m-1)=Pop(2, 1:m);
I_stairs(2:2:2*(m-1))=Pop(2, 1:(m-1));
t_stairs(1:2:2*m-1)=T(1:m);
t_stairs(2:2:2*(m-1))=T(2:m);

figure(1) % evolution of I
plot(t_stairs,I_stairs/N, T_ODE, V, '--'); 
legend('Stochastic', 'Deterministic');
axis([0 T(end) 0 1])

end