function BR
% Computes best response function for a 2 player 2 strategy game.
% Identifies candidates for ESS as strategies p that are best responses
% to itself. It also naively checks what may pass local superiority test.

% One can tweak the payoff function, discretization of the strategy
% space and tolerance for maxima calculations

%% definition of the payoffs
function p=payoff(s1,s2)
    % return payoff to player using s1 if everybody else uses s2
    p = s1.*s2+(1-s1).*(1-s2) ;
end
    
%% user defined parameters 
tol=0.0005; % one may need to test several values of tol
            % too low or too high is not good
n_points=101; % # of points for discretization of the strategy space
              % for example, if suspected ESS is around  1/3, 
              % use n_points = large multiple of 3 + 1
              % to get a good plot even with a low value of tol

%% actual calculations
% Payoff and Best response are summarized in a n_points-by-n_points matrix.
% Each row is P1's mixed strategy, each column is P2's mixed strategy
% BR contains 1 for best response, 0 for not
s = linspace(0,1,n_points); %strategy space
[S1,S2]=meshgrid(s,s);
Payoff = payoff(S1, S2); % find payoff 
P_max = max(Payoff,[],2); % identify maximal payoff to p1 given s2
% identify all candidates (and almost candidates) to be best responses
BR = (P_max(:, ones(n_points,1)) - Payoff < tol);
%% drawing
ms=s(2)-s(1); % meshsize, used for drawing
axis([-ms/2 1+ms/2 -ms/2 1+ms/2 ]);
xlabel('s, representing strategy (1-s,s)')
ylabel('Best response to s')
hold on
% below we will fill a black square for each coordinate that represents
% a best response to a given strategy s1
for xi=1:n_points
    for yi=1:n_points
        if BR(xi,yi) %if s(yi) is a best response to s(xi)
            % get x and y coordinates of the square
            x_coord=[s(xi)-ms/2 s(xi)-ms/2 s(xi)+ms/2 s(xi)+ms/2];
            y_coord=[s(yi)-ms/2 s(yi)+ms/2 s(yi)+ms/2 s(yi)-ms/2];
            fill(x_coord, y_coord, 'black', 'EdgeColor','none');
        end %if BR(xi, yi)
    end; %for yi
end; %for xi

%% outputs
disp('Searching Best Responses to itself ')
for n = 1:n_points
    if BR(n,n)
        disp(['Strategy ', rats(s(n)), ' is a best reponse to itself'])
    end
end

disp(' ')
disp('Testing for local superiority');
% for p to be locally superior, one needs 
% Payoff(p,q)>Payoff(q,q) for all q close to p
% we thus test only the real neighbours
for n = 1:n_points
    if  ((n==1) && (Payoff(1,2)>Payoff(2,2))) || ...
        ((n>1) && (n<n_points) &&  (Payoff(n,n-1)>Payoff(n-1,n-1)) ...
               && (Payoff(n,n+1)>Payoff(n+1,n+1))) || ...
        ((n==n_points) &&  (Payoff(n,n-1)>Payoff(n-1,n-1)))
            disp(['Strategy ', rats(s(n)), ' may be locally superior.'])
    end
end
end